#!/usr/bin/env python

# PyTel command line utility for access .tel data

# Copyright (c) 2009, Andrew McDonald
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#
#    * The name of the author may not be used to endorse or promote
#      products derived from this software without specific prior
#      written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import sys
import os
import getopt
import csv
import getpass
import urllib2
import base64
import M2Crypto
import telsoap
import telquery

rcdir = os.path.expanduser("~/.pytel")

def printusage(argv):
    print "Usage: %s [init|refresh|query|emailquery|snomurl] [options]" % \
        os.path.basename(argv[0])

def printusageinit(argv):
    print "Usage: %s init [options]" % \
        os.path.basename(argv[0])
    print "Options:"
    print "-u, --user=username"
    print "-p, --pass=password"
    print "-c, --challresp=challenge_response"

def printusagerefresh(argv):
    print "Usage: %s refresh" % \
        os.path.basename(argv[0])
    print "No options"

def printusagequery(argv):
    print "Usage: %s query domain [service]" % \
        os.path.basename(argv[0])
    print "domain is a .tel domain"

def printusageemailquery(argv):
    print "Usage: %s emailquery domain" % \
        os.path.basename(argv[0])
    print "domain is a .tel domain"

def printusagesnomurl(argv):
    print "Usage: %s snomurl [options] server" % \
        os.path.basename(argv[0])
    print "server is the web server hosting the telsnom.py cgi script"
    print "Options:"
    print "-u, --user=phone_admin_username"
    print "-p, --pass=phone_admin_password"
    print "-t, --phone=phone_hostname"
    print "-f, --funckey=function_key"
    print "-n, --nopriv"

def doinit(username, password, challresp):
    # initialise config

    if username == None:
        sys.stdout.write("Username: ")
        username = sys.stdin.readline().rstrip()

    if password == None:
        #sys.stdout.write("Password: ")
        #password = sys.stdin.readline().rstrip()
        password = getpass.getpass("Password: ")

    if challresp == None:
        chall = telsoap.getChallenge(username, password)
        if chall == None:
            print "Incorrect username or password"
            return None
        print "Challenge: %s" % chall
        #sys.stdout.write("Response: ")
        #challresp = sys.stdin.readline().rstrip()
        challresp = getpass.getpass("Response: ")

    
    cred = telsoap.getAPICredentials(username, password, challresp)
    if cred == None:
        print "Incorrect challenge response"
        return None

    (soid, api_pass, priv_key) = cred

    if not os.path.isdir(rcdir):
        os.mkdir(rcdir)
        os.chmod(rcdir, 0700)

    f = open(rcdir+'/soapuser', 'w+')
    f.write(soid+'\n')
    f.write(api_pass+'\n')
    f.close()

    priv_key.save_pem(rcdir+'/telkey.pem', None)

    return (soid, api_pass, priv_key)

def loadconf():
    try:
        f = open(rcdir+'/soapuser', 'r')
        soid = f.readline().rstrip()
        api_pass = f.readline().rstrip()
        f.close()
    except:
        soid = None
        api_pass = None

    try:
        priv_key = M2Crypto.RSA.load_key(rcdir+'/telkey.pem',
                                         M2Crypto.util.no_passphrase_callback)
    except:
        priv_key = None

    return (soid, api_pass, priv_key)

def loadfriends():
    friends = {}
    try:
        f = open(rcdir+'/friendlist.csv', 'r')
        csv_reader = csv.reader(f)
        for r in csv_reader:
            friends[r[0]] = r[1]
        f.close()
    except:
        friends = {}
    return friends

def dorefresh(soid, api_pass, priv_key):
    pubs = telsoap.listPublishers(soid, api_pass)
    if pubs == None:
        print "Authentication data may be bad, try reinitializing"
        return None
    if len(pubs) == 1:
        pl = "y"
    else:
        pl = "ies"
    print "Refreshing list with %d entr%s" % (len(pubs), pl)
    f = open(rcdir+'/friendlist.csv', 'w+')
    for (p,l) in pubs:
        f.write("%s,%s\n" % (p,l))
    f.close()

def main(argv):
    if len(argv) < 2 or not argv[1] in ["init","refresh","query",
                                        "emailquery","snomurl"]:
        printusage(argv)
        sys.exit(1)

    if argv[1] == "init":
        username = None
        password = None
        challresp = None
        try:
            (opts,args) = getopt.getopt(argv[2:], "u:p:c:",
                                        ["user=","pass=","challresp="])
        except getopt.GetoptError:
            printusageinit(argv)
            sys.exit(1)

        if len(args) > 0:
            printusage(argv)
            sys.exit(1)

        for (o,v) in opts:
            if o == "-u" or o == "--user":
                username = v
            elif o == "-p" or o == "--pass":
                password = v
            elif o == "-c" or o == "--challresp":
                challresp = v
        cred = doinit(username, password, challresp)
        if cred == None:
            sys.exit(2)
        (soid, api_pass, priv_key) = cred
        dorefresh(soid, api_pass, priv_key)

    elif argv[1] == "refresh":
        if len(argv) > 2:
            printusagerefresh(argv)
            sys.exit(1)
        (soid, api_pass, priv_key) = loadconf()
        if soid == None or api_pass == None or priv_key == None:
            print "Unable to load credentials, try reinitializing"
            sys.exit(2)
        
        dorefresh(soid, api_pass, priv_key)

    elif argv[1] == "query":
        if not (len(argv) >= 3 and len(argv) <= 4) \
                or argv[2][-4:] != ".tel":
            printusagequery(argv)
            sys.exit(1)
        if len(argv) == 4:
            service = argv[3]
        else:
            service = None
        (soid, api_pass, priv_key) = loadconf()
        if priv_key == None:
            # credentials not available
            friends = {}
        else:
            friends = loadfriends()
        telquery.printlist(argv[2], service, priv_key, friends)

    elif argv[1] == "emailquery":
        if not len(argv) == 3 or argv[2][-4:] != ".tel":
            printusageemailquery(argv)
            sys.exit(1)
        (soid, api_pass, priv_key) = loadconf()
        if priv_key == None:
            # credentials not available
            friends = {}
        else:
            friends = loadfriends()
        telquery.printemaillist(argv[2], priv_key, friends)

    elif argv[1] == "snomurl":
        username = None
        password = None
        phone = None
        funckey = None
        nopriv=False
        try:
            (opts,args) = getopt.getopt(argv[2:], "u:p:t:f:n",
                                        ["user=","pass=",
                                         "phone=","funckey=",
                                         "nopriv"])
        except getopt.GetoptError:
            printusagesnomurl(argv)
            sys.exit(1)

        if len(args) != 1:
            printusagesnomurl(argv)
            sys.exit(1)

        for (o,v) in opts:
            if o == "-u" or o == "--user":
                username = v
            elif o == "-p" or o == "--pass":
                password = v
            elif o == "-t" or o == "--phone":
                phone = v
            elif o == "-f" or o == "--funckey":
                funckey = v
            elif o == "-n" or o == "--nopriv":
                nopriv = True

        if phone:
            bad = False
            if not (username and phone and funckey):
                bad = True
            else:
                try:
                    fval = int(funckey)
                    if fval < 1 or fval > 12:
                        bad = True
                except ValueError:
                    bad = True

            if bad:
                printusagesnomurl(argv)
                sys.exit(1)

            funckey = str(fval-1)

        if nopriv:
            (soid, api_pass, priv_key) = (None, None, None)
        else:
            (soid, api_pass, priv_key) = loadconf()

        if priv_key == None:
            # credentials not available
            qstr = ""
        else:
            friends = loadfriends()
            pemkey = priv_key.as_pem(None)
            friendstr = ""
            for d in friends:
                if len(friendstr) > 0:
                    friendstr += ";"
                friendstr += "%s,%s" % (d,friends[d])

            qstr = "?telkey=%s&friends=%s" % (urllib2.quote(pemkey),
                                              urllib2.quote(friendstr))

        cgiurl="http://%s/cgi-bin/telsnom.py%s" % (args[0], qstr)

        print cgiurl

        if phone:
            authdata = base64.encodestring('%s:%s' % (username, password))[:-1]
            authheader =  "Basic %s" % authdata

            funcseturl = "http://%s/dummy.htm?settings=save&fkey%s=%s" % \
                (phone, funckey, urllib2.quote("url "+cgiurl))
            req = urllib2.Request(funcseturl)
            req.add_header("User-Agent", "PyTel/1.0")
            req.add_header("Authorization", authheader)

            try:
                response = urllib2.urlopen(req)
            except urllib2.HTTPError, e:
                if e.code == 401:
                    print "Incorrect phone username or password"
                    return
                else:
                    raise e
            print "Settings pushed to phone"
        else:
            print "Settings must be manually set on phone"

    return

if __name__ == "__main__":
    main(sys.argv)
