#!/usr/bin/python

# CGI script for access .tel domains from the minibrowser
# on Snom SIP phones.

# Copyright (c) 2009, Andrew McDonald
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions
# are met:
#
#    * Redistributions of source code must retain the above copyright
#      notice, this list of conditions and the following disclaimer.
#
#    * Redistributions in binary form must reproduce the above
#      copyright notice, this list of conditions and the following
#      disclaimer in the documentation and/or other materials provided
#      with the distribution.
#
#    * The name of the author may not be used to endorse or promote
#      products derived from this software without specific prior
#      written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
# FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE
# COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
# BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
# LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
# ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
# POSSIBILITY OF SUCH DAMAGE.

import cgi
import telquery
import sys
import os
import urllib2
import M2Crypto

def print_SnomIPPhoneText(search, text):
    print "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
    print "<SnomIPPhoneText>"
    print "<Title>Search %s</Title>" % search
    print "<Prompt>.tel Search</Prompt>"
    print "<Text>%s</Text>" % text
    print "</SnomIPPhoneText>"
    return

def print_SnomIPPhoneInput(server, script, telkey, friends):
    print "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
    print "<SnomIPPhoneInput>"
    print "<Title>.tel search</Title>"
    print "<Prompt>.tel Search</Prompt>"
    print "<URL>http://%s%s</URL>" % (server, script)
    print "<InputItem>"
    print "<DisplayName>.tel Search</DisplayName>"
    if telkey and friends:
        print "<QueryStringParam>telkey=%s&friends=%s&search</QueryStringParam>" % (urllib2.quote(telkey), urllib2.quote(friends))
    else:
        print "<QueryStringParam>search</QueryStringParam>"
    print "<DefaultValue/>"
    print "<InputFlags>l</InputFlags>"
    print "</InputItem>"
    print "</SnomIPPhoneInput>"
    return

def print_SnomIPPhoneDirectory(search, ts):
    print "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
    print "<SnomIPPhoneDirectory>"
    print "<Title>Search %s</Title>" % search
    print "<Prompt>.tel Search</Prompt>"
    for (_,_,t,_) in ts:
        if t[:4] == "tel:":
            t = t[4:]
        print "<DirectoryEntry>"
        print "<Name>%s</Name>" % t
        print "<Telephone>%s</Name>" % t
        print "</DirectoryEntry>"
    print "</SnomIPPhoneDirectory>"
    return

def print_SnomIPPhoneMenu(server, script, telkey, friends, friendlist):
    print "<?xml version=\"1.0\" encoding=\"UTF-8\"?>"
    print "<SnomIPPhoneMenu>"
    print "<Title>.tel Search</Title>"
    for f in sorted(friendlist.keys()):
        print "<MenuItem>"
        print "<Name>%s</Name>" % f
        print "<URL>http://%s%s?telkey=%s&friends=%s&search=%s</QueryStringParam>" % \
            (server, script, urllib2.quote(telkey), urllib2.quote(friends), f)
        print "</MenuItem>"
    print "</SnomIPPhoneMenu>"
    return

def telsnom():
    print "Content-Type: text/xml"     # XML is following
    print                              # blank line, end of headers

    form = cgi.FieldStorage(keep_blank_values=True)

    if "telkey" in form:
        telkey = form["telkey"].value
        if len(telkey) == 0:
            telkey = None
    else:
        telkey = None
    if "friends" in form:
        friends = form["friends"].value
        if len(friends) == 0:
            friends = None
    else:
        friends = None
            
    if not "search" in form or (len(form["search"].value) == 0 and \
                                    friends == None):
        print_SnomIPPhoneInput(os.environ["SERVER_NAME"],
                               os.environ["SCRIPT_NAME"],
                               telkey, friends)
        return

    s = form["search"].value

    friendlist = {}
    if friends:
        fsplit = friends.split(";")
        for f in fsplit:
            f1 = f.split(",")
            friendlist[f1[0]] = f1[1]

    if len(s) == 0:
        if telkey and friends:
            print_SnomIPPhoneMenu(os.environ["SERVER_NAME"],
                                  os.environ["SCRIPT_NAME"],
                                  telkey, friends, friendlist)
        else:
            print_SnomIPPhoneText("all", "No friends.")
        return

    if not form["search"].value[-4:] == ".tel":
        s = form["search"].value+".tel"

    if telkey:
        priv_key = M2Crypto.RSA.load_key_string(telkey,
                                                M2Crypto.util.no_passphrase_callback)
    else:
        priv_key = None

    ts = telquery.query(s, "sip", priv_key, friendlist)
    ts2 = telquery.query(s, "voice:tel", priv_key, friendlist)
    ts += ts2
    ts.sort()

    if len(ts) == 0:
        print_SnomIPPhoneText(s, "No numbers found.")
        return

    print_SnomIPPhoneDirectory(s, ts)

    return


if __name__ == "__main__":
    import cgitb
    cgitb.enable(format="text")

    telsnom()
